/*
 * This file is part of Barista.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Barista is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Barista is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.FileOutputStream;
import java.util.Collections;
import java.util.List;
import static java.util.Arrays.asList;

import fr.x9c.barista.api.*;
import static fr.x9c.barista.api.Instruction.*;
import static tests.Common.*;

public final class Source {

    private static final FieldRef systOut =
	new FieldRef("java.lang.System",
		     "out",
		     new Descriptor.Class("java.io.PrintStream"));

    private static final MethodRef println =
	new MethodRef("java.io.PrintStream",
		      "println",
		      Descriptor.VOID,
		      descs(new Descriptor.Class("java.lang.String")));

    public static void main(final String[] args) throws Throwable {
	final List<Instruction> instructions =
	    asList(new GETSTATIC(systOut),
		   new LDC_STRING("hello"),
		   new INVOKEVIRTUAL(println),
		   RETURN);
	final List<Attribute> methAttrs =
	    asList(new Attribute.Deprecated(),
		   new Attribute.Exceptions(asList("java.io.IOException",
						   "java.lang.RuntimeException")));
	final List<Attribute> attrs =
	    asList(new Attribute.LineNumberTable(asList(0, 3, 5, 8),
						 asList(10, 20, 30, 40)),
		   new Attribute.LocalVariableTable(asList(0),
						    asList(9),
						    asList("args"),
						    descs(new Descriptor.Array(new Descriptor.Class("java.lang.String"))),
						    asList(0)));
	final Method main = compileMethod(asList(AccessFlag.Public, AccessFlag.Static),
					  "main",
					  Descriptor.VOID,
					  descs(new Descriptor.Array(new Descriptor.Class("java.lang.String"))),
					  4,
					  4,
					  Collections.<Attribute.ExceptionTableElement>emptyList(),
					  attrs,
					  methAttrs,
					  instructions);
	final ClassDefinition cd = compileClass(null,
						null,
						null,
						null,
						null,
						null,
						asList(main));
	final byte[] bc = ByteCode.encode(cd);
	FileOutputStream out = new FileOutputStream("pack/Test.class");
	out.write(bc);
	out.flush();
    }

}
